#!/usr/bin/env python
# -*- coding: utf-8 -*-


from __future__ import print_function
from time import sleep
from os import system
from sys import stdout

from daqhats import mcc118, OptionFlags, HatIDs, HatError
from daqhats_utils import select_hat_device, enum_mask_to_string, \
    chan_list_to_mask

from uldaq import get_daq_device_inventory, InterfaceType, ScanStatus, ScanOption, CInScanFlag
from uldaq import CounterMeasurementType, CounterMeasurementMode, CounterEdgeDetection, CounterTickSize
from uldaq import CounterDebounceMode, CounterDebounceTime, CConfigScanFlag, create_int_buffer, DaqDevice

READ_ALL_AVAILABLE = -1

CURSOR_BACK_2 = '\x1b[2D'
ERASE_TO_END_OF_LINE = '\x1b[0K'


def main():
	# ========= USB-QUAD08 variables ========================================
    daq_device = None
    ctr_device = None

    descriptor_index = 0
    interface_type = InterfaceType.USB
    low_encoder = 0
    encoder_count = 2
    sample_rate = 1000.0  # Hz
    samples_per_channel = 10000
    scan_options = ScanOption.CONTINUOUS | ScanOption.EXTCLOCK
    scan_flags = CInScanFlag.DEFAULT

    encoder_type = CounterMeasurementType.ENCODER
    encoder_mode = CounterMeasurementMode.ENCODER_X1 | CounterMeasurementMode.ENCODER_CLEAR_ON_Z
    edge_detection = CounterEdgeDetection.RISING_EDGE
    tick_size = CounterTickSize.TICK_20ns
    debounce_mode = CounterDebounceMode.NONE
    debounce_time = CounterDebounceTime.DEBOUNCE_0ns
    config_flags = CConfigScanFlag.DEFAULT
    # ==========MCC 118 variables ===========================================
    channels = [0, 1, 2, 3]
    channel_mask = chan_list_to_mask(channels)
    num_channels = len(channels)

    hat_samples_per_channel = 0

    options = OptionFlags.CONTINUOUS

    scan_rate = 1000.0
    # ====================================================================
    

    try:
        # Get descriptors for all of the available DAQ devices.
        devices = get_daq_device_inventory(interface_type)
        number_of_devices = len(devices)
        if number_of_devices == 0:
            raise Exception('Error: No DAQ devices found')

        print('Found', number_of_devices, 'DAQ device(s):')
        for i in range(number_of_devices):
            print('    ', devices[i].product_name, ' (', devices[i].unique_id, ')', sep='')

        # Create the DAQ device object associated with the specified descriptor index.
        daq_device = DaqDevice(devices[descriptor_index])

        # Get the CtrDevice object and verify that it is valid.
        ctr_device = daq_device.get_ctr_device()
        if ctr_device is None:
            raise Exception('Error: The DAQ device does not support counters')

        # Verify that the specified device supports hardware pacing for counters.
        ctr_info = ctr_device.get_info()
        if not ctr_info.has_pacer():
            raise Exception('\nError: The specified DAQ device does not support hardware paced counter input')

        # Establish a connection to the DAQ device.
        descriptor = daq_device.get_descriptor()
        print('\nConnecting to', descriptor.dev_string, '- please wait...')
        daq_device.connect()

        # Get the encoder counter channels.
        encoder_counters = get_supported_encoder_counters(ctr_info)
        if len(encoder_counters) == 0:
            raise Exception('\nError: The specified DAQ device does not support encoder channels')

        # Verify that the low_encoder number is valid.
        first_encoder = encoder_counters[0]
        if low_encoder < first_encoder:
            low_encoder = first_encoder

        if low_encoder > first_encoder + len(encoder_counters) - 1:
            low_encoder = first_encoder

        # Verify that the encoder count is valid.
        if encoder_count > len(encoder_counters):
            encoder_count = len(encoder_counters)

        # Set the high_encoder channel.
        high_encoder = low_encoder + encoder_count - 1
        if high_encoder > first_encoder + len(encoder_counters) - 1:
            high_encoder = first_encoder + len(encoder_counters) - 1

        # update the actual number of encoders being used
        encoder_count = high_encoder - low_encoder + 1

        # Clear the counter, and configure the counter as an encoder.
        for encoder in range(low_encoder, high_encoder + 1):
            ctr_device.c_config_scan(encoder, encoder_type, encoder_mode, edge_detection, tick_size,
                                     debounce_mode, debounce_time, config_flags)

        # Allocate a buffer to receive the data.
        data = create_int_buffer(encoder_count, samples_per_channel)

        print('\n', descriptor.dev_string, ' ready', sep='')
        print('    Function demonstrated: ctr_device.c_config_scan()')
        print('    Counter(s):', low_encoder, '-', high_encoder)
        print('    Sample rate:', sample_rate, 'Hz')
        print('    Scan options:', display_scan_options(scan_options))
        # ============================end quad08 setup========================
        
        # Select an MCC 118 HAT device to use.
        address = select_hat_device(HatIDs.MCC_118)
        hat = mcc118(address)

        print('\nFound MCC 118 HAT device at address', address)

        actual_scan_rate = hat.a_in_scan_actual_rate(num_channels, scan_rate)

        print('\nMCC 118 continuous scan example')
        print('    Functions demonstrated:')
        print('         mcc118.a_in_scan_start')
        print('         mcc118.a_in_scan_read')
        print('         mcc118.a_in_scan_stop')
        print('    Channels: ', end='')
        print(', '.join([str(chan) for chan in channels]))
        print('    Requested scan rate: ', scan_rate)
        print('    Actual scan rate: ', actual_scan_rate)
        print('    Options: ', enum_mask_to_string(OptionFlags, options))
        total_samples_read = 0
        read_request_size = READ_ALL_AVAILABLE
        timeout = 5.0
        
        print('\nPlease connect MCC 118 CLK to USB-QUAD08 XPCR. ')
        
        # ==================end mcc 118 setup===============================
        try:
            input('\nHit ENTER to continue\n')
        except (NameError, SyntaxError):
            pass

        # Start the scan
      
        ctr_device.c_in_scan(low_encoder, high_encoder, samples_per_channel,
                             sample_rate, scan_options, scan_flags, data)
                             
		# Configure and start the scan.
        # Since the continuous option is being used, the hat_samples_per_channel
        # parameter is ignored if the value is less than the default internal
        # buffer size (10000 * num_channels in this case). If a larger internal
        # buffer size is desired, set the value of this parameter accordingly.
        hat.a_in_scan_start(channel_mask, hat_samples_per_channel, scan_rate, options)
        sleep(0.1)

        system('clear')

        try:
            while True:
				try:
                
					status, transfer_status = ctr_device.get_scan_status()

					reset_cursor()
					print('Please enter CTRL + C to terminate the process\n')

					print('quad08 actual scan rate = ', '{:.6f}'.format(sample_rate), 'Hz\n')

					index = transfer_status.current_index
					print('quad08 currentScanCount = ', transfer_status.current_scan_count)
					print('quad08 currentTotalCount = ', transfer_status.current_total_count)
					print('quad08 currentIndex = ', index, '\n')

					for encoder_index in range(encoder_count):
						print('chan =', (encoder_index + low_encoder), ': ', '{:6d}'.format(data[index + encoder_index]))
						
					print('\n')


					read_result = hat.a_in_scan_read(read_request_size, timeout)

					# Check for an overrun error
					if read_result.hardware_overrun:
						print('\n\nHardware overrun\n')
						break
					elif read_result.buffer_overrun:
						print('\n\nBuffer overrun\n')
						break
			
					samples_read_per_channel = int(len(read_result.data) / num_channels)
					total_samples_read += samples_read_per_channel
					print('hat samples per channel:', samples_read_per_channel)

					# Display the last sample for each channel.
					print('\r{:12}'.format(samples_read_per_channel),
						  ' {:12} '.format(total_samples_read), end='')

					if samples_read_per_channel > 0:
						index = samples_read_per_channel * num_channels - num_channels

					for i in range(num_channels):
						print('{:10.5f}'.format(read_result.data[index+i]), 'V ',
							  end='')
					stdout.flush()
					sleep(0.1)

					if status != ScanStatus.RUNNING:
						break
				except (ValueError, NameError, SyntaxError):
					break
        except KeyboardInterrupt:
            pass

    except Exception as e:
        print('\n', e)

    finally:
        if daq_device:
            if ctr_device:
                ctr_device.scan_stop()
            if daq_device.is_connected():
                daq_device.disconnect()
            daq_device.release()


def display_scan_options(bit_mask):
    options = []
    if bit_mask == ScanOption.DEFAULTIO:
        options.append(ScanOption.DEFAULTIO.name)
    for so in ScanOption:
        if so & bit_mask:
            options.append(so.name)
    return ', '.join(options)


def get_supported_encoder_counters(ctr_info):
    encoders = []

    number_of_counters = ctr_info.get_num_ctrs()
    for counter_number in range(number_of_counters):
        measurement_types = ctr_info.get_measurement_types(counter_number)

        if CounterMeasurementType.ENCODER in measurement_types:
            encoders.append(counter_number)

    return encoders


def reset_cursor():
    stdout.write('\033[1;1H')


if __name__ == '__main__':
    main()
